//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
//
//	sdrc_lite sdr sdram controller
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
`timescale 1ns/100ps
module SDRC_CMD_FSM(
	sys_clk,
	sys_rst_n,
	sdrc_sclr_n,
	sys_rd_req,
	sys_wr_req,
	sdrc_ref_req,
	bl_cnt_num,
	cmd_cnt_num,
	cmd_nop,
	cmd_ref,
	cmd_act,
	cmd_wra,
	cmd_rda,
	cmd_ready,
	bl_cnt_sclr,
	c_cmd_cnt_sclr,	
	dp_rd,
	dp_wr
);
`include "../rtl/SDRC_LITE_PAR.v"
	//interface signals
	input	sys_clk;
	input	sys_rst_n;
	input	sdrc_sclr_n;
	input	sys_rd_req;
	input	sys_wr_req;
	input	sdrc_ref_req;
	input	[BL_CNT_W-1:0]	bl_cnt_num;
	input	[CMD_CNT_W-1:0]	cmd_cnt_num;
	output	reg cmd_nop;
	output	reg cmd_ref;
	output	reg cmd_act;
	output	reg cmd_wra;
	output	reg cmd_rda;
	output	reg cmd_ready;
	output	reg bl_cnt_sclr;
	output	reg c_cmd_cnt_sclr;
	output	reg dp_rd;
	output	reg dp_wr;
	//internal regs
	reg		[3:0]	c_st_now;
	reg		[3:0]	c_st_nxt;
	reg				fsm_rd_wr_n;
	//states
	parameter	c_st_nop	= 4'b0000;
	parameter	c_st_ref	= 4'b0001;
	parameter	c_st_trfc	= 4'b0010;
	parameter	c_st_act	= 4'b0011;
	parameter	c_st_trcd	= 4'b0100;
	parameter	c_st_rda	= 4'b0101;
	parameter	c_st_cl		= 4'b0110;
	parameter	c_st_rdat	= 4'b0111;
	parameter	c_st_wra	= 4'b1000;
	parameter	c_st_wdat	= 4'b1001;
	parameter	c_st_tdal	= 4'b1010;
	parameter	c_st_nop0	= 4'b1011;
	//request latch
	always@(posedge sys_clk, negedge sys_rst_n)
		if(sys_rst_n == 1'b0)
			fsm_rd_wr_n	<=	1'b0;
		else if(sdrc_sclr_n == 1'b0)
			fsm_rd_wr_n	<=	1'b0;
		else if(c_st_now == c_st_nop)
			fsm_rd_wr_n	<=	sys_rd_req;
	//load next state
	always@(posedge sys_clk, negedge sys_rst_n)
		if(sys_rst_n == 1'b0)
			c_st_now	<=	c_st_nop;
		else if(sdrc_sclr_n == 1'b0)
			c_st_now	<=	c_st_nop;
		else
			c_st_now	<=	c_st_nxt;
	//next state logic and output logic
	always @ (c_st_now, sys_rd_req, sys_wr_req, sdrc_ref_req, bl_cnt_num, 
		cmd_cnt_num, fsm_rd_wr_n) begin
		cmd_ready		= 1'b0;
		cmd_nop			= 1'b0;
		cmd_ref			= 1'b0;
		cmd_act			= 1'b0;
		cmd_wra			= 1'b0;
		cmd_rda			= 1'b0;
		bl_cnt_sclr		= 1'b0;		
		c_cmd_cnt_sclr	= 1'b0;
		dp_rd			= 1'b0;
		dp_wr			= 1'b0;
		case(c_st_now)
			//To enable tdal delay overlap, 
			//	we must make sure FSM stays idle for at least 2 clks. [1]
			c_st_nop0:	begin
				cmd_nop			= 1'b1;
				cmd_ready		= 1'b1;
				c_st_nxt		= c_st_nop;				
			end
			c_st_nop:	begin
				cmd_nop			= 1'b1;
				cmd_ready		= 1'b1;
				if(sdrc_ref_req == 1'b1)
					c_st_nxt	= c_st_ref;
				else if((sys_rd_req == 1'b1) | (sys_wr_req == 1'b1))
					c_st_nxt	= c_st_act;
				else
					c_st_nxt	= c_st_nop;
			end
			c_st_ref:	begin
				cmd_ref			= 1'b1;
				c_cmd_cnt_sclr	= 1'b1;
				if(CtRFCm1 < 3)						//[1]
					c_st_nxt	= c_st_nop0;
				else
					c_st_nxt	= c_st_trfc;
			end
			c_st_trfc:	begin
				cmd_nop			= 1'b1;
				if(cmd_cnt_num == CtRFCm1 - 3)		//[1]
					c_st_nxt	= c_st_nop;
				else
					c_st_nxt	= c_st_trfc;
			end
			c_st_act:	begin
				cmd_act			= 1'b1;
				c_cmd_cnt_sclr	= 1'b1;
				if(CtRCDm1 == 0)
					c_st_nxt	= (fsm_rd_wr_n) ? c_st_rda : c_st_wra;
				else
					c_st_nxt	= c_st_trcd;
			end
			c_st_trcd:	begin
				cmd_nop			= 1'b1;
				if(cmd_cnt_num == CtRCDm1 - 1)
					c_st_nxt	= (fsm_rd_wr_n) ? c_st_rda : c_st_wra;
				else
					c_st_nxt	= c_st_trcd;
			end
			c_st_rda:	begin
				cmd_rda			= 1'b1;
				bl_cnt_sclr		= 1'b1;
				c_st_nxt		= c_st_cl;
			end
			c_st_cl:	begin
				cmd_nop			= 1'b1;
				if(bl_cnt_num == pCL - 2)
					c_st_nxt	= c_st_rdat;
				else
					c_st_nxt	= c_st_cl;	
			end
			c_st_rdat:	begin
				cmd_nop			= 1'b1;
				dp_rd			= 1'b1;
				if(bl_cnt_num == pCL + pBL - 2)			//[1]
					c_st_nxt	= c_st_nop;
				else
					c_st_nxt	= c_st_rdat;
			end
			c_st_wra:	begin
				cmd_wra			= 1'b1;
				bl_cnt_sclr		= 1'b1;
				c_st_nxt		= c_st_wdat;
			end
			c_st_wdat:	begin
				cmd_nop			= 1'b1;
				dp_wr			= 1'b1;
				if(bl_cnt_num == pBL - 1)
					if(CtDALm1 < 3)						//[1]
						c_st_nxt	= c_st_nop0;
					else
						c_st_nxt	= c_st_tdal;						
				else
					c_st_nxt	= c_st_wdat;
			end
			c_st_tdal:	begin
				cmd_nop			= 1'b1;
				//CtDALm1=2,1,0 excluded
				if(bl_cnt_num == pBL - 1 + CtDALm1 - 2)	//[1]
					c_st_nxt	= c_st_nop0;
				else
					c_st_nxt	= c_st_tdal;	
			end			
			default:	begin							//[2]
				cmd_nop			= 1'b1;
				c_st_nxt		= c_st_nop;
			end
		endcase
	end
endmodule		
/*
Tricks & Questions:
[1]	Cmd_fsm	will stay in idle(c_st_nop) state for at least 2 cycles.During the 
		1st cycle in idle state, cmd_ready changes to 1. During the 2rd cycle 
		in idle state, bus master knows cmd_ready is 1 and might possilby send 
		a request. If so, command fsm will leave idle state in the next cycle. 
	These 2 cycles can be counted in CtDAL during write transaction. And they 
		can cover additional waiting cycles during read transaction 
		if (CtRP-1)>(CL-1).
	I will discuss this using FSM waveform and device autoprecharge diagram in
		my project document.
[2] Warning: Jump to c_st_nop may not satisfy AC timing.
*/
