//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
//
//	sdrc_lite sdr sdram controller
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
`timescale 1ns/100ps
module SDRC_CTRL(
	sys_clk,
	sys_rst_n,
	sys_sclr_n,
	sys_req,
	sys_ref,
	sys_wr_n,
	sys_busy,
	ini_nop,
	ini_prea,
	ini_ref,
	ini_lmr,
	ini_ready,
	cmd_nop,
	cmd_ref,
	cmd_act,
	cmd_wra,
	cmd_rda,
	cmd_ready,
	sys_rd_req,
	sys_wr_req,
	dp_rd,
	dp_wr
);
`include "../rtl/SDRC_LITE_PAR.v"
//system bus interface
input					sys_clk;			// system clock
input					sys_rst_n;			// system reset
input					sys_sclr_n;			// system synchronous clear
input					sys_req;			// system request
input					sys_ref;			// system refresh request
input					sys_wr_n;			// 0: write; 1: read.
output					sys_busy;			// sdrc ctrl busy
//signal path interface		
output					ini_nop;			// ini fsm no operation
output					ini_prea;			// ini fsm precharge all
output					ini_ref;			// ini fsm refresh
output					ini_lmr;			// ini fsm mode register set
output					ini_ready;			// ini fsm initialization over
output					cmd_nop;			// cmd fsm no operation
output					cmd_ref;			// cmd fsm refresh
output					cmd_act;			// cmd fsm row activate
output					cmd_wra;			// cmd fsm write with autoprecharge
output					cmd_rda;			// cmd fsm read with autoprecharge
output					cmd_ready;			// cmd fsm idle
//data path interface
output					sys_rd_req;			// read request to cmd fsm
output					sys_wr_req;			// write request to cmd fsm
output					dp_rd;				// datapath read
output					dp_wr;				// datapath write
//internal signals
wire					sdrc_sclr_n; 		// ini_ready & sys_sclr_n
wire					ref_cnt_ref_req;	// refresh counter refresh request
wire					sdrc_ref_req;		// refesh request to cmd fsm
wire					ini_wait_done;		// 100us/200us wait done
wire					i_ref_over;			// 2/8 initial refresh done
wire	[CMD_CNT_W-1:0]	cmd_cnt_num;		// command delay counter num
wire					i_cmd_cnt_sclr;		// cmd cnt clear from ini fsm
wire					c_cmd_cnt_sclr;		// cmd cnt clear from cmd fsm
wire	[BL_CNT_W-1:0]	bl_cnt_num;			// write/read control counter num
wire					bl_cnt_sclr;		// bl cnt clear
assign sdrc_sclr_n = ini_ready & sys_sclr_n;
SDRC_REQ_MAN sdrc_req_man0(
	.sys_req(sys_req),
	.sys_ref(sys_ref),
	.sys_wr_n(sys_wr_n),
	.ref_cnt_ref_req(ref_cnt_ref_req),
	.ini_ready(ini_ready),
	.cmd_ready(cmd_ready),
	.sys_busy(sys_busy),
	.sdrc_ref_req(sdrc_ref_req),
	.sys_rd_req(sys_rd_req),
	.sys_wr_req(sys_wr_req)
);
SDRC_INI_FSM sdrc_ini_fsm0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sys_sclr_n(sys_sclr_n),
	.ini_wait_done(ini_wait_done),
	.i_ref_over(i_ref_over),
	.cmd_cnt_num(cmd_cnt_num),
	.i_cmd_cnt_sclr(i_cmd_cnt_sclr),
	.ini_nop(ini_nop),
	.ini_prea(ini_prea),
	.ini_ref(ini_ref),
	.ini_lmr(ini_lmr),
	.ini_ready(ini_ready)
);
SDRC_CMD_FSM sdrc_cmd_fsm0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sdrc_sclr_n(sdrc_sclr_n),
	.sys_rd_req(sys_rd_req),
	.sys_wr_req(sys_wr_req),
	.sdrc_ref_req(sdrc_ref_req),
	.bl_cnt_num(bl_cnt_num),
	.cmd_cnt_num(cmd_cnt_num),
	.cmd_nop(cmd_nop),
	.cmd_ref(cmd_ref),
	.cmd_act(cmd_act),
	.cmd_wra(cmd_wra),
	.cmd_rda(cmd_rda),
	.cmd_ready(cmd_ready),
	.bl_cnt_sclr(bl_cnt_sclr),
	.c_cmd_cnt_sclr(c_cmd_cnt_sclr),
	.dp_rd(dp_rd),
	.dp_wr(dp_wr)
);
SDRC_INI_CNT sdrc_ini_cnt0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sys_sclr_n(sys_sclr_n),
	.ini_wait_done(ini_wait_done)
);
SDRC_REF_N_CNT sdrc_ref_n_cnt0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sys_sclr_n(sys_sclr_n),
	.ini_ref(ini_ref),
	.i_ref_over(i_ref_over)
);
SDRC_CMD_CNT sdrc_cmd_cnt0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sys_sclr_n(sys_sclr_n),
	.i_cmd_cnt_sclr(i_cmd_cnt_sclr),
	.c_cmd_cnt_sclr(c_cmd_cnt_sclr),
	.cmd_cnt_num(cmd_cnt_num)
);
SDRC_BL_CNT sdrc_bl_cnt0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sdrc_sclr_n(sdrc_sclr_n),
	.bl_cnt_sclr(bl_cnt_sclr),
	.bl_cnt_num(bl_cnt_num)
);
SDRC_REF_CNT sdrc_ref_cnt0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sdrc_sclr_n(sdrc_sclr_n),
	.cmd_ref(cmd_ref),
	.ref_cnt_ref_req(ref_cnt_ref_req)
);
endmodule
