//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
//
//	sdrc_lite sdr sdram controller
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
`timescale 1ns/100ps
module SDRC_CTRL_tb;
`include "../rtl/SDRC_LITE_PAR.v"
//system bus interface
reg						sys_clk;			// system clock
reg						sys_rst_n;			// system reset
reg						sys_sclr_n;			// system synchronous clear
reg						sys_req;			// system request
reg						sys_ref;			// system refresh request
reg						sys_wr_n;			// 0: write; 1: read.
wire					sys_busy;			// sdrc ctrl busy
//signal path interface		
wire					ini_nop;			// ini fsm no operation
wire					ini_prea;			// ini fsm precharge all
wire					ini_ref;			// ini fsm refresh
wire					ini_lmr;			// ini fsm mode register set
wire					ini_ready;			// ini fsm initialization over
wire					cmd_nop;			// cmd fsm no operation
wire					cmd_ref;			// cmd fsm refresh
wire					cmd_act;			// cmd fsm row activate
wire					cmd_wra;			// cmd fsm write with autoprecharge
wire					cmd_rda;			// cmd fsm read with autoprecharge
wire					cmd_ready;			// cmd fsm idle
//data path interface
wire					sys_rd_req;			// read request to cmd fsm
wire					sys_wr_req;			// write request to cmd fsm
wire					dp_rd;				// datapath read
wire					dp_wr;				// datapath write
//test varibles
integer					cnt_t;
//sdrc_ctrl instance
SDRC_CTRL sdrc_ctrl0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sys_sclr_n(sys_sclr_n),
	.sys_req(sys_req),
	.sys_ref(sys_ref),
	.sys_wr_n(sys_wr_n),
	.sys_busy(sys_busy),
	.ini_nop(ini_nop),
	.ini_prea(ini_prea),
	.ini_ref(ini_ref),
	.ini_lmr(ini_lmr),
	.ini_ready(ini_ready),
	.cmd_nop(cmd_nop),
	.cmd_ref(cmd_ref),
	.cmd_act(cmd_act),
	.cmd_wra(cmd_wra),
	.cmd_rda(cmd_rda),
	.cmd_ready(cmd_ready),
	.sys_rd_req(sys_rd_req),
	.sys_wr_req(sys_wr_req),
	.dp_rd(dp_rd),
	.dp_wr(dp_wr)
);
//clk
initial sys_clk = 1'b0;
always #5 sys_clk = ~sys_clk;
//rst and conditions
initial begin
	sys_rst_n		= 1'b1;
	sys_sclr_n		= 1'b1;
	sys_req			= 1'b0;
	sys_ref			= 1'b0;
	sys_wr_n		= 1'b0;
	#1
	sys_rst_n 		= 1'b0;
	#1
	sys_rst_n 		= 1'b1;
	//initial wait time
	#(CtINIw*10)
	//initialize FSM operate and wait
	#(CtRFCm1*10*200)
	//wait
	#20
	for(cnt_t = 1; cnt_t < 100; cnt_t = cnt_t+1) begin
		write;
	end
	#1000
	for(cnt_t = 1; cnt_t < 100; cnt_t = cnt_t+1) begin
		read;
	end
	#1000
	refresh;
	refresh;
	write;
	write;
	read;
	read;
	write;
	refresh;
	read;
	refresh;
	#1000
	$finish;
end
//For Verdi or Debussy debugging
initial begin
	$fsdbDumpfile("SDRC_CTRL_tb.fsdb");
	$fsdbDumpvars(0,SDRC_CTRL_tb);
end
//transaction definitions
//task write
task write;
//	input	[SYS_A_W-1:0]	sys_addr;
//	input	[SYS_D_W-1:0]	sys_wr_dat;
	begin
		while(sys_busy == 1'b1) begin
			#10
			sys_req			= 1'b0;
			sys_ref			= 1'b0;
			sys_wr_n		= 1'b0;			
		end
		#10
		sys_req			= 1'b1;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
		#10
		sys_req			= 1'b0;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
/*
		#(CtRCDm1*10)
		#10
		#(pBL*10)
		#(CtDALm1*10)
		sys_req			= 1'b0;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
*/	
	end
endtask
//task read
task read;
//	input	[SYS_A_W-1:0]	sys_addr;
//	input	[SYS_D_W-1:0]	sys_rd_dat;
	begin
		while(sys_busy == 1'b1) begin
			#10
			sys_req			= 1'b0;
			sys_ref			= 1'b0;
			sys_wr_n		= 1'b0;			
		end
		#10
		sys_req			= 1'b1;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b1;
		#10
		sys_req			= 1'b0;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
/*
		#(CtRCDm1*10)
		#10
		#(pCL*10)
		#(pBL*10)
		sys_req			= 1'b0;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
*/
	end
endtask
//task refresh
task refresh;
	begin
		while(sys_busy == 1'b1) begin
			#10
			sys_req			= 1'b0;
			sys_ref			= 1'b0;
			sys_wr_n		= 1'b0;			
		end
		#10
		sys_req			= 1'b1;
		sys_ref			= 1'b1;
		sys_wr_n		= 1'b0;
		#10
		sys_req			= 1'b0;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
	end
endtask
endmodule
