//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
//
//	sdrc_lite sdr sdram controller
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
`timescale 1ns/100ps
module SDRC_INI_FSM(
	sys_clk,
	sys_rst_n,
	sys_sclr_n,
	ini_wait_done,
	i_ref_over,
	cmd_cnt_num,
	i_cmd_cnt_sclr,
	ini_nop,
	ini_prea,
	ini_ref,
	ini_lmr,
	ini_ready
);
`include "../rtl/SDRC_LITE_PAR.v"
	//interface signals
	input	sys_clk;
	input	sys_rst_n;
	input	sys_sclr_n;
	input	ini_wait_done;
	input	i_ref_over;
	input	[CMD_CNT_W-1:0]	cmd_cnt_num;
	output	reg	i_cmd_cnt_sclr;
	output	reg ini_nop;
	output	reg ini_prea;
	output	reg ini_ref;
	output	reg ini_lmr;
	output	reg ini_ready;
	//internal signals
	reg		[2:0]	i_st_now;
	reg		[2:0]	i_st_nxt;
	//states
	parameter	i_st_nop	= 3'b000;
	parameter	i_st_prea	= 3'b001;
	parameter	i_st_trp	= 3'b010;
	parameter	i_st_ref	= 3'b011;
	parameter	i_st_trfc	= 3'b100;
	parameter	i_st_lmr	= 3'b101;
	parameter	i_st_tmrd	= 3'b110;
	parameter	i_st_ready	= 3'b111;
	//load next state
	always@(posedge sys_clk, negedge sys_rst_n)
		if(sys_rst_n == 1'b0)
			i_st_now	<=	i_st_nop;
		else if(sys_sclr_n == 1'b0)
			i_st_now	<=	i_st_nop;
		else
			i_st_now	<=	i_st_nxt;
	//next state logic and output logic
	always @ (i_st_now, ini_wait_done, i_ref_over, cmd_cnt_num) begin
		ini_nop			= 1'b0;
		ini_prea		= 1'b0;
		ini_ref			= 1'b0;
		ini_lmr			= 1'b0;
		ini_ready		= 1'b0;
		i_cmd_cnt_sclr	= 1'b0;
		case(i_st_now)
			i_st_nop:	begin
				ini_nop			= 1'b1;
				if(ini_wait_done == 1'b1)
					i_st_nxt	= i_st_prea;
				else
					i_st_nxt	= i_st_nop;
			end
			i_st_prea:	begin
				ini_prea		= 1'b1;
				i_cmd_cnt_sclr	= 1'b1;
				if(CtRPm1 == 0)
					i_st_nxt	= i_st_ref;
				else
					i_st_nxt	= i_st_trp;
			end
			i_st_trp:	begin
				ini_nop			= 1'b1;
				if(cmd_cnt_num == CtRPm1 - 1)
					i_st_nxt	= i_st_ref;
				else
					i_st_nxt	= i_st_trp;
			end
			i_st_ref:	begin
				ini_ref			= 1'b1;
				i_cmd_cnt_sclr	= 1'b1;
				if(CtRFCm1 == 0)
					i_st_nxt	= (i_ref_over) ? i_st_lmr : i_st_ref;
				else
					i_st_nxt	= i_st_trfc;
			end
			i_st_trfc:	begin
				ini_nop			= 1'b1;
				if(cmd_cnt_num == CtRFCm1 - 1)
					i_st_nxt	= (i_ref_over) ? i_st_lmr : i_st_ref;
				else
					i_st_nxt	= i_st_trfc;				
			end
			i_st_lmr:	begin
				ini_lmr			= 1'b1;
				i_cmd_cnt_sclr	= 1'b1;
				if(CtMRDm1 == 0)
					i_st_nxt	= i_st_ready;
				else
					i_st_nxt	= i_st_tmrd;
			end
			i_st_tmrd:	begin
				ini_nop			= 1'b1;
				if(cmd_cnt_num == CtMRDm1 - 1)
					i_st_nxt	= i_st_ready;
				else
					i_st_nxt	= i_st_tmrd;
			end
			i_st_ready:	begin
				ini_ready		= 1'b1;
				i_st_nxt		= i_st_ready;
			end
			default:	begin
				ini_nop			= 1'b1;
				i_st_nxt		= i_st_nop;
			end
		endcase
	end
endmodule
