//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
//
//	sdrc_lite sdr sdram controller
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\

//64Mbit SDR SDRAM ports spec
parameter	SDR_A_W			=	12;					//address port width
parameter	SDR_R_W			=	12;					//2^12 = 4096  rows
parameter	SDR_C_W			=	8;					//2^8  = 256   columns
parameter	SDR_D_W			=	16;					//data port width
parameter	SDR_M_W			=	2;					//data mask port width

//System Frequency
parameter	integer tCK		=	10;					//ns, 100MHz

//Refrash interval
//All rows should be covered in 64ms for standard sdr sdram
//All rows should be covered in 16ms for automobile sdr sdram
parameter	integer tREFi	=	64000000/(2**SDR_R_W);	//ns
parameter	integer CtREFi	=	0.9*tREFi/tCK;		//clks, 0.9 for security
parameter	REFi_CNT_W		=	log2(CtREFi);		//bits

//Power on initialization
//8 refr & 200us wait in JEDEC/PC100
//2 refr & 100us wait for Micron/ISSI
//8 refr & 200us wait for Hynix/Elpida/WinBond/Nanya
parameter	tINIw			=	200000;				//ns
parameter	integer CtINIw	=	1.1*tINIw/tCK;		//clks, 1.1 for security
parameter	INIw_CNT_W		=	log2(CtINIw);		//bits
parameter	IrefN			=	8;					//times
parameter	IrefN_CNT_W		=	log2(IrefN);		//bits

//Samsung K4S641632N-LC60 SDR SDRAM AC spec (CL=3) in ns
parameter	tMRD			=	2*tCK;				//De facto standard
parameter	tRP				=	18;					//ns
parameter	tRFC			=	60;					//ns, usually equal to tRC
parameter	tRCD			=	18;					//ns
parameter	tWR				=	2*tCK;				//ns, a.k.a tDPL
parameter	tDAL			=	tWR + tRP;			//ns

//AC spec in clk cycles
parameter	integer CtMRDm1	=	div(tMRD,tCK) - 1;	//clks
parameter	integer CtRPm1	=	div(tRP,tCK) - 1;	//clks
parameter	integer CtRFCm1	=	div(tRFC,tCK) - 1;	//clks
parameter	integer CtRCDm1	=	div(tRCD,tCK) - 1;	//clks
parameter	integer CtWRm1	=	div(tWR,tCK) - 1;	//clks
parameter	integer CtDALm1	=	div(tDAL,tCK) - 1;	//clks

//Mode register parameters
parameter	integer pCL		=	3;
parameter	integer pBL		=	8;
parameter	rsv_val		=	{(SDR_A_W-10){1'b1}};
parameter	MR_val		=	{rsv_val, 1'b0, 2'b00, 3'b011, 1'b0 ,3'b011};

//Command counter width
parameter	CMD_CNT_W		=	log2(CtRFCm1);		//bits

//Command data counter width
parameter	BL_CNT_W		=	log2(max(pCL+pBL-2, pBL+CtDALm1-1)); //bits

//Functions
//log2 and ceiling it
function integer log2(input integer n);
	integer i;
	begin
		log2 = 1;
		for(i=0; 2**i<n; i = i + 1)
			log2 = i + 1;
	end
endfunction
//div and ceiling it
function integer div(input integer a, input integer b);
	integer i;
	begin
		div = 1;
		for(i=0; b*i<a; i = i + 1)
			div = i + 1;
	end
endfunction
//max of two numbers
function integer max(input integer s, input integer t);
	begin
		max = s;
		if(t > s)
			max = t;
	end
endfunction
