//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
//
//	sdrc_lite sdr sdram controller
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
`timescale 1ns/100ps
module SDRC_SIG_PATH_FF(
	sys_clk,
	sys_rst_n,
	sys_sclr_n,
	sys_addr,
	sys_rd_req,
	sys_wr_req,
	ini_prea,
	ini_ref,
	ini_lmr,
	ini_ready,
	cmd_ref,
	cmd_act,
	cmd_wra,
	cmd_rda,
	sdr_cke,
	sdr_cs_n,
	sdr_ras_n,
	sdr_cas_n,
	sdr_we_n,
	sdr_dqm,
	sdr_ba,
	sdr_addr
);
`include "../rtl/SDRC_LITE_PAR.v"
//interface to system bus
input					sys_clk;			// system clock
input					sys_rst_n;			// system reset
input					sys_sclr_n;			// system synchronous clear
input	[SYS_A_W-1:0]	sys_addr;			// system address
//interface to sdrc_ctrl
input					sys_rd_req;			// read request to cmd fsm
input					sys_wr_req;			// write request to cmd fsm
input					ini_prea;			// ini fsm precharge all
input					ini_ref;			// ini fsm refresh
input					ini_lmr;			// ini fsm mode register set
input					ini_ready;			// ini fsm initialization over
input					cmd_ref;			// cmd fsm refresh
input					cmd_act;			// cmd fsm row activate
input					cmd_wra;			// cmd fsm write with autoprecharge
input					cmd_rda;			// cmd fsm read with autoprecharge
// interface to sdram
output					sdr_cke;			// sdr clock enable
output					sdr_cs_n;			// sdr chip select
output					sdr_ras_n;			// sdr row address strobe
output					sdr_cas_n;			// sdr column address strobe
output					sdr_we_n;			// sdr write enable
output	[SDR_M_W-1:0]	sdr_dqm;			// sdr data mask
output	[SDR_B_W-1:0]	sdr_ba;				// sdr bank address bus
output	[SDR_A_W-1:0]	sdr_addr;			// sdr address bus
// internal wires
wire	[SDR_B_W-1:0]	sys_ba;
wire	[SDR_R_W-1:0]	sys_ra;
wire	[SDR_C_W-1:0]	sys_ca;
reg		[3:0]			sdr_cmd_nxt;
reg		[SDR_B_W-1:0]	sdr_ba_nxt;
reg		[SDR_A_W-1:0]	sdr_addr_nxt;
// internal registers
reg		[SDR_B_W-1:0]	sys_ba_rg;
reg		[SDR_R_W-1:0]	sys_ra_rg;
reg		[SDR_C_W-1:0]	sys_ca_rg;
reg		[3:0]			sdr_cmd_rg;
reg		[SDR_B_W-1:0]	sdr_ba_rg;
reg		[SDR_A_W-1:0]	sdr_addr_rg;
// sdram command parameter {sdr_cs_n, sdr_ras_n, sdr_cas_n, sdr_we_n}
parameter	SDR_LMR		= 4'b0000;			// sdr load mode register
parameter	SDR_REF		= 4'b0001;			// sdr refresh
parameter	SDR_PRE		= 4'b0010;			// sdr precharge
parameter	SDR_ACT		= 4'b0011;			// sdr row activate
parameter	SDR_WR		= 4'b0100;			// sdr column write
parameter	SDR_RD		= 4'b0101;			// sdr column read
parameter	SDR_BT		= 4'b0110;			// sdr burst terminate
parameter	SDR_NOP		= 4'b0111;			// sdr no operation
parameter	SDR_DSEL	= 4'b1000;			// sdr deselect
// memory address map
assign {sys_ba, sys_ra, sys_ca} = sys_addr;
// system address register
always@(posedge sys_clk, negedge sys_rst_n)
	if(sys_rst_n == 1'b0) begin
		sys_ba_rg		<= 0;
		sys_ra_rg		<= 0;
		sys_ca_rg		<= 0;
	end
	else if(sys_sclr_n == 1'b0) begin
		sys_ba_rg		<= 0;
		sys_ra_rg		<= 0;
		sys_ca_rg		<= 0;
	end
	else if((sys_rd_req == 1'b1) | (sys_wr_req == 1'b1)) begin
		sys_ba_rg		<= sys_ba;
		sys_ra_rg		<= sys_ra;
		sys_ca_rg		<= sys_ca;
	end
// sdram clock enable output
assign sdr_cke = 1'b1;
// sdram data bus mask output
assign sdr_dqm = ~{SDR_M_W{ini_ready}};
// sdram command/address outputs
assign {sdr_cs_n, sdr_ras_n, sdr_cas_n, sdr_we_n} = sdr_cmd_rg;
assign sdr_ba = sdr_ba_rg;
assign sdr_addr = sdr_addr_rg;
// sdram command/address register
always@(posedge sys_clk, negedge sys_rst_n)
	if(sys_rst_n == 1'b0) begin
		sdr_ba_rg		<= 0;
		sdr_addr_rg		<= 0;
		sdr_cmd_rg		<= SDR_NOP;
	end
	else if(sys_sclr_n == 1'b0) begin
		sdr_ba_rg		<= 0;
		sdr_addr_rg		<= 0;
		sdr_cmd_rg		<= SDR_NOP;
	end
	else begin
		sdr_ba_rg		<= sdr_ba_nxt;
		sdr_addr_rg		<= sdr_addr_nxt;
		sdr_cmd_rg		<= sdr_cmd_nxt;	
	end
// sdram nxt command/address generate
always@({ini_prea, ini_ref, ini_lmr, cmd_ref, cmd_act, cmd_wra, cmd_rda,
	sys_ba_rg, sys_ra_rg, sys_ca_rg})
	case({ini_prea, ini_ref, ini_lmr, cmd_ref, cmd_act, cmd_wra, cmd_rda})
		7'b100_0000: 	begin			//ini_prea
			sdr_ba_nxt		= 0;
			sdr_addr_nxt	= {SDR_A_W{1'b1}};		//A10 should be 1
			sdr_cmd_nxt		= SDR_PRE;
		end
		7'b010_0000: 	begin			//ini_ref
			sdr_ba_nxt		= 0;
			sdr_addr_nxt	= 0;
			sdr_cmd_nxt		= SDR_REF;			
		end
		7'b001_0000: 	begin			//ini_lmr
			sdr_ba_nxt		= 0;
			sdr_addr_nxt	= MR_val;
			sdr_cmd_nxt		= SDR_LMR;			
		end
		7'b000_1000: 	begin			//cmd_ref
			sdr_ba_nxt		= 0;
			sdr_addr_nxt	= 0;
			sdr_cmd_nxt		= SDR_REF;			
		end
		7'b000_0100: 	begin			//cmd_act
			sdr_ba_nxt		= sys_ba_rg;
			sdr_addr_nxt	= sys_ra_rg;
			sdr_cmd_nxt		= SDR_ACT;
		end
		7'b000_0010: 	begin			//cmd_wra
			sdr_ba_nxt		= sys_ba_rg;
			sdr_addr_nxt	= col_addr_gen(sys_ca_rg, 1'b1);
			sdr_cmd_nxt		= SDR_WR;
		end
		7'b000_0001: 	begin			//cmd_rda
			sdr_ba_nxt		= sys_ba_rg;
			sdr_addr_nxt	= col_addr_gen(sys_ca_rg, 1'b1);
			sdr_cmd_nxt		= SDR_RD;
		end
		default: 		begin			//nop
			sdr_ba_nxt		= 0;
			sdr_addr_nxt	= 0;
			sdr_cmd_nxt		= SDR_NOP;
		end
	endcase
//function: column address gen
function [SDR_A_W-1:0] col_addr_gen(input [SDR_C_W-1:0] ca, input [0:0] a10);
	reg		[SDR_A_W-2:0]	ca_ext;
	begin
		ca_ext				= {{(SDR_A_W-SDR_C_W-1){1'b0}},ca};
		col_addr_gen		= {ca_ext[SDR_A_W-2:10], a10, ca_ext[9:0]};
	end
endfunction
endmodule
/*
For use of generate-conditional, refer to IEEE Std 1364-2001 p177
*/
