//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
//
//	sdrc_lite sdr sdram controller
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
`timescale 1ns/100ps
module SDRC_SIG_PATH_FF_TST(
	sys_clk,
	sys_rst_n,
	sys_sclr_n,
	sys_req,
	sys_ref,
	sys_wr_n,
	sys_busy,
	sys_addr,
	sdr_cke,
	sdr_cs_n,
	sdr_ras_n,
	sdr_cas_n,
	sdr_we_n,
	sdr_dqm,
	sdr_ba,
	sdr_addr
);
`include "../rtl/SDRC_LITE_PAR.v"
// system bus interface
input					sys_clk;			// system clock
input					sys_rst_n;			// system reset
input					sys_sclr_n;			// system synchronous clear
input					sys_req;			// system request
input					sys_ref;			// system refresh request
input					sys_wr_n;			// 0: write; 1: read.
output					sys_busy;			// sdrc ctrl busy
input	[SYS_A_W-1:0]	sys_addr;			// system address
// interface to sdram
output					sdr_cke;			// sdr clock enable
output					sdr_cs_n;			// sdr chip select
output					sdr_ras_n;			// sdr row address strobe
output					sdr_cas_n;			// sdr column address strobe
output					sdr_we_n;			// sdr write enable
output	[SDR_M_W-1:0]	sdr_dqm;			// sdr data mask
output	[SDR_B_W-1:0]	sdr_ba;				// sdr bank address bus
output	[SDR_A_W-1:0]	sdr_addr;			// sdr address bus
// internal signal path & data path signals
wire					ini_nop;			// ini fsm no operation
wire					ini_prea;			// ini fsm precharge all
wire					ini_ref;			// ini fsm refresh
wire					ini_lmr;			// ini fsm mode register set
wire					ini_ready;			// ini fsm initialization over
wire					cmd_nop;			// cmd fsm no operation
wire					cmd_ref;			// cmd fsm refresh
wire					cmd_act;			// cmd fsm row activate
wire					cmd_wra;			// cmd fsm write with autoprecharge
wire					cmd_rda;			// cmd fsm read with autoprecharge
wire					cmd_ready;			// cmd fsm idle
wire					sys_rd_req;			// read request to cmd fsm
wire					sys_wr_req;			// write request to cmd fsm
wire					dp_rd;				// datapath read
wire					dp_wr;				// datapath writ
SDRC_CTRL sdrc_ctrl0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sys_sclr_n(sys_sclr_n),
	.sys_req(sys_req),
	.sys_ref(sys_ref),
	.sys_wr_n(sys_wr_n),
	.sys_busy(sys_busy),
	.ini_nop(ini_nop),
	.ini_prea(ini_prea),
	.ini_ref(ini_ref),
	.ini_lmr(ini_lmr),
	.ini_ready(ini_ready),
	.cmd_nop(cmd_nop),
	.cmd_ref(cmd_ref),
	.cmd_act(cmd_act),
	.cmd_wra(cmd_wra),
	.cmd_rda(cmd_rda),
	.cmd_ready(cmd_ready),
	.sys_rd_req(sys_rd_req),
	.sys_wr_req(sys_wr_req),
	.dp_rd(dp_rd),
	.dp_wr(dp_wr)
);
// SDRC_SIG_PATH_FF instance
SDRC_SIG_PATH_FF sdrc_sig_path_ff0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sys_sclr_n(sys_sclr_n),
	.sys_addr(sys_addr),
	.sys_rd_req(sys_rd_req),
	.sys_wr_req(sys_wr_req),
	.ini_prea(ini_prea),
	.ini_ref(ini_ref),
	.ini_lmr(ini_lmr),
	.ini_ready(ini_ready),
	.cmd_ref(cmd_ref),
	.cmd_act(cmd_act),
	.cmd_wra(cmd_wra),
	.cmd_rda(cmd_rda),
	.sdr_cke(sdr_cke),
	.sdr_cs_n(sdr_cs_n),
	.sdr_ras_n(sdr_ras_n),
	.sdr_cas_n(sdr_cas_n),
	.sdr_we_n(sdr_we_n),
	.sdr_dqm(sdr_dqm),
	.sdr_ba(sdr_ba),
	.sdr_addr(sdr_addr)
);
endmodule
