//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
//
//	sdrc_lite sdr sdram controller
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
`timescale 1ns/100ps
module SDRC_SIG_PATH_FF_TST_tb;
`include "../rtl/SDRC_LITE_PAR.v"
//interface to system bus
reg						sys_clk;			// system clock
reg						sys_rst_n;			// system reset
reg						sys_sclr_n;			// system synchronous clear
reg						sys_req;			// system request
reg						sys_ref;			// system refresh request
reg						sys_wr_n;			// 0: write; 1: read.
wire					sys_busy;			// sdrc ctrl busy
reg		[SYS_A_W-1:0]	sys_addr;			// system address
// interface to sdram
wire					sdr_cke;			// sdr clock enable
wire					sdr_cs_n;			// sdr chip select
wire					sdr_ras_n;			// sdr row address strobe
wire					sdr_cas_n;			// sdr column address strobe
wire					sdr_we_n;			// sdr write enable
wire	[SDR_M_W-1:0]	sdr_dqm;			// sdr data mask
wire	[SDR_B_W-1:0]	sdr_ba;				// sdr bank address bus
wire	[SDR_A_W-1:0]	sdr_addr;			// sdr address bus
// testbench variable
reg		[SDR_A_W*4-1:0] cnt_t;
// SDRC_SIG_PATH_FF instance
SDRC_SIG_PATH_FF_TST sdrc_sig_path_ff_tst0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sys_sclr_n(sys_sclr_n),
	.sys_req(sys_req),
	.sys_ref(sys_ref),
	.sys_wr_n(sys_wr_n),
	.sys_busy(sys_busy),
	.sys_addr(sys_addr),
	.sdr_cke(sdr_cke),
	.sdr_cs_n(sdr_cs_n),
	.sdr_ras_n(sdr_ras_n),
	.sdr_cas_n(sdr_cas_n),
	.sdr_we_n(sdr_we_n),
	.sdr_dqm(sdr_dqm),
	.sdr_ba(sdr_ba),
	.sdr_addr(sdr_addr)
);
//clk
initial sys_clk = 1'b0;
always #5 sys_clk = ~sys_clk;
//rst and conditions
initial begin
	sys_rst_n		= 1'b1;
	sys_sclr_n		= 1'b1;
	sys_req			= 1'b0;
	sys_ref			= 1'b0;
	sys_wr_n		= 1'b0;
	sys_addr		= 0;
	#1
	sys_rst_n 		= 1'b0;
	#1
	sys_rst_n 		= 1'b1;
	//initial wait time
	#(CtINIw*10)
	//initialize FSM operate and wait
	#(CtRFCm1*10*200)
	//wait
	#20
	for(cnt_t = 1; cnt_t < 100; cnt_t = cnt_t+1) begin
		write(cnt_t[SDR_A_W-1:0]);
	end
	#1000
	for(cnt_t = 1; cnt_t < 100; cnt_t = cnt_t+1) begin
		read(cnt_t[SDR_A_W-1:0]);
	end
	#1000
	refresh;
	refresh;
	write(sys_addr_gen(1,4,7));
	write(sys_addr_gen(2,3,24));
	read(sys_addr_gen(0,14,73));
	read(sys_addr_gen(3,114,5));
	write(sys_addr_gen(2,89,31));
	refresh;
	read(sys_addr_gen(2,53,82));
	refresh;
	#500
	$finish;		
end
//For Verdi or Debussy debugging
initial begin
	$fsdbDumpfile("SDRC_SIG_PATH_FF_TST_tb.fsdb");
	$fsdbDumpvars(0,SDRC_SIG_PATH_FF_TST_tb);
end
//transaction definitions
//task write
task write;
	input	[SYS_A_W-1:0]	tsk_sys_addr;
//	input	[SYS_D_W-1:0]	tsk_sys_wr_dat;
	begin
		while(sys_busy == 1'b1) begin
			#10
			sys_req			= 1'b0;
			sys_ref			= 1'b0;
			sys_wr_n		= 1'b0;			
		end
		#10
		sys_req			= 1'b1;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
		sys_addr		= tsk_sys_addr;
		#10
		sys_req			= 1'b0;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
/*
		#(CtRCDm1*10)
		#10
		#(pBL*10)
		#(CtDALm1*10)
		sys_req			= 1'b0;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
*/	
	end
endtask
//task read
task read;
	input	[SYS_A_W-1:0]	tsk_sys_addr;
//	input	[SYS_D_W-1:0]	tsk_sys_rd_dat;
	begin
		while(sys_busy == 1'b1) begin
			#10
			sys_req			= 1'b0;
			sys_ref			= 1'b0;
			sys_wr_n		= 1'b0;			
		end
		#10
		sys_req			= 1'b1;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b1;
		sys_addr		= tsk_sys_addr;
		#10
		sys_req			= 1'b0;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
/*
		#(CtRCDm1*10)
		#10
		#(pCL*10)
		#(pBL*10)
		sys_req			= 1'b0;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
*/
	end
endtask
//task refresh
task refresh;
	begin
		while(sys_busy == 1'b1) begin
			#10
			sys_req			= 1'b0;
			sys_ref			= 1'b0;
			sys_wr_n		= 1'b0;			
		end
		#10
		sys_req			= 1'b1;
		sys_ref			= 1'b1;
		sys_wr_n		= 1'b0;
		#10
		sys_req			= 1'b0;
		sys_ref			= 1'b0;
		sys_wr_n		= 1'b0;
	end
endtask
//function: system address gen
function [SYS_A_W-1:0] sys_addr_gen;
	input	integer			ba_int;
	input	integer			ra_int;
	input	integer			ca_int;
	begin
		//suppose {sys_ba, sys_ra, sys_ca} = sys_addr;
		sys_addr_gen[SYS_A_W-1:SYS_A_W-SDR_B_W]	= ba_int;
		sys_addr_gen[SYS_A_W-SDR_B_W-1:SDR_C_W]	= ra_int;
		sys_addr_gen[SDR_C_W-1:0]				= ca_int;
	end
endfunction
endmodule
