//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
//
//	sdrc_lite sdr sdram controller
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
`timescale 1ns/100ps
module SDRC_SIG_PATH_FF_tb;
`include "../rtl/SDRC_LITE_PAR.v"
//interface to system bus
reg						sys_clk;			// system clock
reg						sys_rst_n;			// system reset
reg						sys_sclr_n;			// system synchronous clear
reg		[SYS_A_W-1:0]	sys_addr;			// system address
//interface to sdrc_ctrl
reg						sys_rd_req;			// read request to cmd fsm
reg						sys_wr_req;			// write request to cmd fsm
reg						ini_prea;			// ini fsm precharge all
reg						ini_ref;			// ini fsm refresh
reg						ini_lmr;			// ini fsm mode register set
reg						ini_ready;			// ini fsm initialization over
reg						cmd_ref;			// cmd fsm refresh
reg						cmd_act;			// cmd fsm row activate
reg						cmd_wra;			// cmd fsm write with autoprecharge
reg						cmd_rda;			// cmd fsm read with autoprecharge
// interface to sdram
wire					sdr_cke;			// sdr clock enable
wire					sdr_cs_n;			// sdr chip select
wire					sdr_ras_n;			// sdr row address strobe
wire					sdr_cas_n;			// sdr column address strobe
wire					sdr_we_n;			// sdr write enable
wire	[SDR_M_W-1:0]	sdr_dqm;			// sdr data mask
wire	[SDR_B_W-1:0]	sdr_ba;				// sdr bank address bus
wire	[SDR_A_W-1:0]	sdr_addr;			// sdr address bus
// SDRC_SIG_PATH_FF instance
SDRC_SIG_PATH_FF sdrc_sig_path_ff0(
	.sys_clk(sys_clk),
	.sys_rst_n(sys_rst_n),
	.sys_sclr_n(sys_sclr_n),
	.sys_addr(sys_addr),
	.sys_rd_req(sys_rd_req),
	.sys_wr_req(sys_wr_req),
	.ini_prea(ini_prea),
	.ini_ref(ini_ref),
	.ini_lmr(ini_lmr),
	.ini_ready(ini_ready),
	.cmd_ref(cmd_ref),
	.cmd_act(cmd_act),
	.cmd_wra(cmd_wra),
	.cmd_rda(cmd_rda),
	.sdr_cke(sdr_cke),
	.sdr_cs_n(sdr_cs_n),
	.sdr_ras_n(sdr_ras_n),
	.sdr_cas_n(sdr_cas_n),
	.sdr_we_n(sdr_we_n),
	.sdr_dqm(sdr_dqm),
	.sdr_ba(sdr_ba),
	.sdr_addr(sdr_addr)
);
//clk
initial sys_clk = 1'b0;
always #5 sys_clk = ~sys_clk;
//rst and conditions
initial begin
	sys_rst_n		= 1'b1;
	sys_sclr_n		= 1'b1;
	sys_addr		= 0;
	sys_rd_req		= 1'b0;
	sys_wr_req		= 1'b0;
	ini_prea		= 1'b0;
	ini_ref			= 1'b0;
	ini_lmr			= 1'b0;
	ini_ready		= 1'b0;
	cmd_ref			= 1'b0;
	cmd_act			= 1'b0;
	cmd_wra			= 1'b0;
	cmd_rda			= 1'b0;
	#1
	sys_rst_n 		= 1'b0;
	#1
	sys_rst_n 		= 1'b1;
	#20
	#10
	ini_prea		= 1'b1;
	#10
	ini_prea		= 1'b0;
	#(CtRPm1*10)
	ini_ref			= 1'b1;
	#10
	ini_ref			= 1'b0;
	#(CtRFCm1*10)
	ini_lmr			= 1'b1;
	#10
	ini_lmr			= 1'b0;
	#(CtMRDm1*10)
	ini_ready		= 1'b1;
	#20
	#10
	cmd_ref			= 1'b1;
	#10
	cmd_ref			= 1'b0;
	#30
	sys_rd_req		= 1'b1;
	sys_addr		= {{SDR_B_W{1'b1}},{SDR_R_W{1'b1}},{SDR_C_W{1'b0}}};
	#10
	sys_rd_req		= 1'b0;
	cmd_act			= 1'b1;
	#10
	cmd_act			= 1'b0;
	#10
	cmd_rda			= 1'b1;
	#10
	cmd_rda			= 1'b0;
	#80
	sys_wr_req		= 1'b1;
	sys_addr		= {{SDR_B_W{1'b1}},{SDR_R_W{1'b1}},{SDR_C_W{1'b0}}};
	#10
	sys_rd_req		= 1'b0;
	cmd_act			= 1'b1;
	#10
	cmd_act			= 1'b0;
	#10
	cmd_wra			= 1'b1;
	#10
	cmd_wra			= 1'b0;
	#80
	$finish;		
end
//For Verdi or Debussy debugging
initial begin
	$fsdbDumpfile("SDRC_SIG_PATH_FF_tb.fsdb");
	$fsdbDumpvars(0,SDRC_SIG_PATH_FF_tb);
end
endmodule
